import { SIZEOF } from "@thi.ng/api/typedarray";
import { align as $align } from "@thi.ng/binary/align";
import { ceilPow2 } from "@thi.ng/binary/pow";
import { isStruct, isUnion } from "./utils.js";
/**
 * C ABI compatible alignment
 */
export const ALIGN_C = {
    align: (field) => {
        let align = SIZEOF[field.type];
        if (field.tag === "vec") {
            align *= ceilPow2(field.len);
        }
        return align;
    },
    size: (size, align) => $align(size, align),
    offset: (offset, align) => $align(offset, align),
};
export const ALIGN_PACKED = {
    align: () => 1,
    size: (size) => size,
    offset: (offset) => offset,
};
/**
 * Returns a suitable alignment strategy for given type, i.e. either the user
 * supplied impl defined for the type or the default ({@link ALIGN_C}).
 *
 * @param type
 */
export const selectAlignment = (type) => isStruct(type) || isUnion(type) ? type.align || ALIGN_C : ALIGN_C;
